@@ -24,18 +24,22 @@
 using namespace qtgmenu;
 
 QtGMenuImporter::QtGMenuImporter( const QString& service, const QDBusObjectPath& menu_path,
-                                  const QString& action_prefix, const QDBusObjectPath& action_path, QObject* parent )
+                                  const QString& action_prefix, const QDBusObjectPath& action_path,
+                                  const QDBusConnection& connection, QSharedPointer<GDBusConnection> gconnection,
+                                  QObject* parent )
     : QObject( parent )
 {
     QMap<QString, QDBusObjectPath> action_paths;
     action_paths[action_prefix] = action_path;
-    d.reset( new QtGMenuImporterPrivate( service, menu_path, action_paths, *this ) );
+    d.reset( new QtGMenuImporterPrivate( service, menu_path, action_paths, *this, connection, gconnection ) );
 }
 
 QtGMenuImporter::QtGMenuImporter( const QString& service, const QDBusObjectPath& menu_path,
-                                  const QMap<QString, QDBusObjectPath>& action_paths, QObject* parent )
+                                  const QMap<QString, QDBusObjectPath>& action_paths,
+                                  const QDBusConnection& connection, QSharedPointer<GDBusConnection> gconnection,
+                                  QObject* parent )
     : QObject( parent ),
-      d( new QtGMenuImporterPrivate( service, menu_path, action_paths, *this ) )
+      d( new QtGMenuImporterPrivate( service, menu_path, action_paths, *this, connection, gconnection ) )
 {
 }
 
